@@ -44,20 +44,10 @@ struct _ClutterBackend
   CoglRenderer *cogl_renderer;
   CoglDisplay *cogl_display;
   CoglContext *cogl_context;
-  GSource *cogl_source;
-
-  CoglOnscreen *dummy_onscreen;
 
   ClutterDeviceManager *device_manager;
 
-  cairo_font_options_t *font_options;
-
-  gchar *font_name;
-
-  gfloat units_per_em;
-  gint32 units_serial;
-
-  GList *event_translators;
+  ClutterBackendPrivate *priv;
 };
 
 struct _ClutterBackendClass
@@ -102,16 +92,12 @@ struct _ClutterBackendClass
                                                 gpointer            native,
                                                 ClutterEvent       *event);
 
-  PangoDirection        (* get_keymap_direction) (ClutterBackend   *backend);
-
   /* signals */
   void (* resolution_changed) (ClutterBackend *backend);
   void (* font_changed)       (ClutterBackend *backend);
   void (* settings_changed)   (ClutterBackend *backend);
 };
 
-ClutterBackend *        _clutter_create_backend                         (void);
-
 ClutterStageWindow *    _clutter_backend_create_stage                   (ClutterBackend         *backend,
                                                                          ClutterStage           *wrapper,
                                                                          GError                **error);
@@ -149,11 +135,6 @@ gfloat                  _clutter_backend_get_units_per_em               (Clutter
                                                                          PangoFontDescription   *font_desc);
 gint32                  _clutter_backend_get_units_serial               (ClutterBackend         *backend);
 
-PangoDirection          _clutter_backend_get_keymap_direction           (ClutterBackend         *backend);
-
-void                    _clutter_backend_reset_cogl_framebuffer         (ClutterBackend         *backend);
-
-
 G_END_DECLS
 
 #endif /* __CLUTTER_BACKEND_PRIVATE_H__ */
