@@ -34,8 +34,10 @@
 #include <glib.h>
 #include <gmodule.h>
 
+#define CLUTTER_DISABLE_DEPRECATION_WARNINGS
+#include "deprecated/clutter-container.h"
+
 #include "clutter-actor.h"
-#include "clutter-container.h"
 #include "clutter-debug.h"
 #include "clutter-enum-types.h"
 
@@ -43,6 +45,8 @@
 #include "clutter-script-private.h"
 #include "clutter-scriptable.h"
 
+#include "clutter-stage-manager.h"
+
 #include "clutter-private.h"
 
 static void clutter_script_parser_object_end (JsonParser *parser,
@@ -1830,20 +1834,19 @@ _clutter_script_check_unresolved (ClutterScript *script,
       ClutterActor *parent;
 
       parent = clutter_actor_get_parent (CLUTTER_ACTOR (oinfo->object));
-      if (parent != NULL && CLUTTER_IS_CONTAINER (parent))
+      if (parent != NULL)
         {
           ClutterContainer *container = CLUTTER_CONTAINER (parent);
-          GList *children, *l;
+          ClutterActor *child;
 
-          children = clutter_container_get_children (container);
-
-          for (l = children; l != NULL; l = l->next)
+          for (child = clutter_actor_get_first_child (parent);
+               child != NULL;
+               child = clutter_actor_get_next_sibling (child))
             {
-              GObject *child = l->data;
               ObjectInfo *child_info;
               const gchar *id_;
 
-              id_ = clutter_get_script_id (child);
+              id_ = clutter_get_script_id (G_OBJECT (child));
               if (id_ == NULL || *id_ == '\0')
                 continue;
 
@@ -1852,14 +1855,12 @@ _clutter_script_check_unresolved (ClutterScript *script,
                 continue;
 
               apply_child_properties (script, container,
-                                      CLUTTER_ACTOR (child),
+                                      child,
                                       child_info);
               apply_layout_properties (script, container,
-                                       CLUTTER_ACTOR (child),
+                                       child,
                                        child_info);
             }
-
-          g_list_free (children);
         }
     }
 
@@ -1937,7 +1938,7 @@ void
 _clutter_script_construct_object (ClutterScript *script,
                                   ObjectInfo    *oinfo)
 {
-  GArray *params;
+  GArray *params = NULL;
   guint i;
 
   /* we have completely updated the object */
@@ -1966,7 +1967,9 @@ _clutter_script_construct_object (ClutterScript *script,
 
   if (oinfo->is_stage && oinfo->is_stage_default)
     {
+      ClutterStageManager *manager = clutter_stage_manager_get_default ();
       GList *properties = oinfo->properties;
+      ClutterStage *default_stage;
 
       /* the default stage is a complex beast: we cannot create it using
        * g_object_newv() but we need clutter_script_construct_parameters()
@@ -1981,7 +1984,8 @@ _clutter_script_construct_object (ClutterScript *script,
                                              properties,
                                              &params);
 
-      oinfo->object = G_OBJECT (clutter_stage_get_default ());
+      default_stage = clutter_stage_manager_get_default_stage (manager);
+      oinfo->object = G_OBJECT (default_stage);
 
       for (i = 0; i < params->len; i++)
         {
@@ -1996,6 +2000,7 @@ _clutter_script_construct_object (ClutterScript *script,
   else
     {
       GList *properties = oinfo->properties;
+      GParameter *parameters;
 
       /* every other object: first, we get the construction parameters */
       oinfo->properties =
@@ -2005,9 +2010,10 @@ _clutter_script_construct_object (ClutterScript *script,
                                              properties,
                                              &params);
 
+      parameters = (GParameter *) (void *) params->data;
       oinfo->object = g_object_newv (oinfo->gtype,
                                      params->len,
-                                     (GParameter *) params->data);
+                                     parameters);
 
       /* by sinking the floating reference, we make sure that the reference
        * count is correct whether the object is referenced from somewhere
