@@ -44,7 +44,9 @@
 #include "clutter-input-device-evdev.h"
 #include "clutter-main.h"
 #include "clutter-private.h"
+#include "clutter-stage-manager.h"
 #include "clutter-xkb-utils.h"
+#include "clutter-backend-private.h"
 
 #include "clutter-device-manager-evdev.h"
 
@@ -83,9 +85,9 @@ static const gchar *subsystems[] = { "input", NULL };
  */
 
 
-const char *option_xkb_layout = "us";
-const char *option_xkb_variant = "";
-const char *option_xkb_options = "";
+static const char *option_xkb_layout = "us";
+static const char *option_xkb_variant = "";
+static const char *option_xkb_options = "";
 
 /*
  * ClutterEventSource for reading input devices
@@ -151,16 +153,21 @@ notify_key (ClutterEventSource *source,
             guint32             key,
             guint32             state)
 {
+  ClutterInputDevice *input_device = (ClutterInputDevice *) source->device;
+  ClutterStage *stage;
   ClutterEvent *event = NULL;
-  ClutterActor *stage;
 
-  stage = clutter_stage_get_default ();
+  /* We can drop the event on the floor if no stage has been
+   * associated with the device yet. */
+  stage = _clutter_input_device_get_stage (input_device);
+  if (!stage)
+    return;
 
   /* if we have a mapping for that device, use it to generate the event */
   if (source->xkb)
     event =
-      _clutter_key_event_new_from_evdev ((ClutterInputDevice *) source->device,
-                                         CLUTTER_STAGE (stage),
+      _clutter_key_event_new_from_evdev (input_device,
+                                         stage,
                                          source->xkb,
                                          time_, key, state,
                                          &source->modifier_state);
@@ -175,13 +182,19 @@ notify_motion (ClutterEventSource *source,
                gint                x,
                gint                y)
 {
+  ClutterInputDevice *input_device = (ClutterInputDevice *) source->device;
   gfloat stage_width, stage_height, new_x, new_y;
   ClutterEvent *event;
-  ClutterActor *stage;
+  ClutterStage *stage;
 
-  stage = clutter_stage_get_default ();
-  stage_width = clutter_actor_get_width (stage);
-  stage_height = clutter_actor_get_height (stage);
+  /* We can drop the event on the floor if no stage has been
+   * associated with the device yet. */
+  stage = _clutter_input_device_get_stage (input_device);
+  if (!stage)
+    return;
+
+  stage_width = clutter_actor_get_width (CLUTTER_ACTOR (stage));
+  stage_height = clutter_actor_get_height (CLUTTER_ACTOR (stage));
 
   event = clutter_event_new (CLUTTER_MOTION);
 
@@ -203,8 +216,8 @@ notify_motion (ClutterEventSource *source,
   source->y = new_y;
 
   event->motion.time = time_;
-  event->motion.stage = CLUTTER_STAGE (stage);
-  event->motion.device = (ClutterInputDevice *) source->device;
+  event->motion.stage = stage;
+  event->motion.device = input_device;
   event->motion.modifier_state = source->modifier_state;
   event->motion.x = new_x;
   event->motion.y = new_y;
@@ -218,8 +231,9 @@ notify_button (ClutterEventSource *source,
                guint32             button,
                guint32             state)
 {
+  ClutterInputDevice *input_device = (ClutterInputDevice *) source->device;
   ClutterEvent *event;
-  ClutterActor *stage;
+  ClutterStage *stage;
   gint button_nr;
   static gint maskmap[8] =
     {
@@ -227,7 +241,11 @@ notify_button (ClutterEventSource *source,
       CLUTTER_BUTTON4_MASK, CLUTTER_BUTTON5_MASK, 0, 0, 0
     };
 
-  stage = clutter_stage_get_default ();
+  /* We can drop the event on the floor if no stage has been
+   * associated with the device yet. */
+  stage = _clutter_input_device_get_stage (input_device);
+  if (!stage)
+    return;
 
   button_nr = button - BTN_LEFT + 1;
   if (G_UNLIKELY (button_nr < 1 || button_nr > 8))
@@ -264,13 +282,17 @@ clutter_event_dispatch (GSource     *g_source,
                         gpointer     user_data)
 {
   ClutterEventSource *source = (ClutterEventSource *) g_source;
+  ClutterInputDevice *input_device = (ClutterInputDevice *) source->device;
   struct input_event ev[8];
   ClutterEvent *event;
   gint len, i, dx = 0, dy = 0;
   uint32_t _time;
+  ClutterStage *stage;
 
   clutter_threads_enter ();
 
+  stage = _clutter_input_device_get_stage (input_device);
+
   /* Don't queue more events if we haven't finished handling the previous batch
    */
   if (!clutter_events_pending ())
@@ -303,6 +325,10 @@ clutter_event_dispatch (GSource     *g_source,
          goto out;
        }
 
+       /* Drop events if we don't have any stage to forward them to */
+       if (!stage)
+         goto out;
+
        for (i = 0; i < len / sizeof (ev[0]); i++)
          {
            struct input_event *e = &ev[i];
@@ -452,17 +478,8 @@ clutter_event_source_new (ClutterInputDeviceEvdev *input_device)
     }
   else if (type == CLUTTER_POINTER_DEVICE)
     {
-      /* initialize the pointer position to the center of the default stage */
-      ClutterActor *stage;
-      gfloat stage_width, stage_height;
-
-      stage = clutter_stage_get_default ();
-
-      stage_width = clutter_actor_get_width (stage);
-      stage_height = clutter_actor_get_height (stage);
-
-      event_source->x = (gint) stage_width / 2;
-      event_source->y = (gint) stage_height / 2;
+      event_source->x = 0;
+      event_source->y = 0;
     }
 
   /* and finally configure and attach the GSource */
@@ -530,13 +547,11 @@ evdev_add_device (ClutterDeviceManagerEvdev *manager_evdev,
   ClutterDeviceManager *manager = (ClutterDeviceManager *) manager_evdev;
   ClutterInputDeviceType type = CLUTTER_EXTENSION_DEVICE;
   ClutterInputDevice *device;
-  ClutterActor *stage;
-  const gchar *device_file, *sysfs_path;
-  const gchar * const *keys;
-  guint i;
+  const gchar *device_file, *sysfs_path, *device_name;
 
   device_file = g_udev_device_get_device_file (udev_device);
   sysfs_path = g_udev_device_get_sysfs_path (udev_device);
+  device_name = g_udev_device_get_name (udev_device);
 
   if (device_file == NULL || sysfs_path == NULL)
     return;
@@ -550,38 +565,30 @@ evdev_add_device (ClutterDeviceManagerEvdev *manager_evdev,
   if (!is_evdev (sysfs_path))
     return;
 
-  keys = g_udev_device_get_property_keys (udev_device);
-  for (i = 0; keys[i]; i++)
-    {
-      /* Clutter assumes that device types are exclusive in the
-       * ClutterInputDevice API */
-      if (strcmp (keys[i], "ID_INPUT_KEY") == 0)
-        type = CLUTTER_KEYBOARD_DEVICE;
-      else if (strcmp (keys[i], "ID_INPUT_MOUSE") == 0)
-        type = CLUTTER_POINTER_DEVICE;
-      else if (strcmp (keys[i], "ID_INPUT_JOYSTICK") == 0)
-        type = CLUTTER_JOYSTICK_DEVICE;
-      else if (strcmp (keys[i], "ID_INPUT_TABLET") == 0)
-        type = CLUTTER_TABLET_DEVICE;
-      else if (strcmp (keys[i], "ID_INPUT_TOUCHPAD") == 0)
-        type = CLUTTER_TOUCHPAD_DEVICE;
-      else if (strcmp (keys[i], "ID_INPUT_TOUCHSCREEN") == 0)
-        type = CLUTTER_TOUCHSCREEN_DEVICE;
-    }
+  /* Clutter assumes that device types are exclusive in the
+   * ClutterInputDevice API */
+  if (g_udev_device_has_property (udev_device, "ID_INPUT_KEYBOARD"))
+    type = CLUTTER_KEYBOARD_DEVICE;
+  else if (g_udev_device_has_property (udev_device, "ID_INPUT_MOUSE"))
+    type = CLUTTER_POINTER_DEVICE;
+  else if (g_udev_device_has_property (udev_device, "ID_INPUT_JOYSTICK"))
+    type = CLUTTER_JOYSTICK_DEVICE;
+  else if (g_udev_device_has_property (udev_device, "ID_INPUT_TABLET"))
+    type = CLUTTER_TABLET_DEVICE;
+  else if (g_udev_device_has_property (udev_device, "ID_INPUT_TOUCHPAD"))
+    type = CLUTTER_TOUCHPAD_DEVICE;
+  else if (g_udev_device_has_property (udev_device, "ID_INPUT_TOUCHSCREEN"))
+    type = CLUTTER_TOUCHSCREEN_DEVICE;
 
   device = g_object_new (CLUTTER_TYPE_INPUT_DEVICE_EVDEV,
                          "id", 0,
-                         "name", "Evdev device", /* FIXME */
+                         "name", device_name,
                          "device-type", type,
                          "sysfs-path", sysfs_path,
                          "device-path", device_file,
                          "enabled", TRUE,
                          NULL);
 
-  /* Always associate the device to the default stage */
-  stage = clutter_stage_get_default ();
-  _clutter_input_device_set_stage (device, CLUTTER_STAGE (stage));
-
   _clutter_device_manager_add_device (manager, device);
 
   CLUTTER_NOTE (EVENT, "Added device %s, type %d, sysfs %s",
@@ -852,25 +859,18 @@ clutter_device_manager_evdev_init (ClutterDeviceManagerEvdev *self)
   self->priv = CLUTTER_DEVICE_MANAGER_EVDEV_GET_PRIVATE (self);
 }
 
-/*
- * _clutter_events_evdev_init() and _clutter_events_evdev_uninit() are the two
- * symbol to use the evdev event backend from the EGL backend
- */
-
 void
 _clutter_events_evdev_init (ClutterBackend *backend)
 {
   CLUTTER_NOTE (EVENT, "Initializing evdev backend");
 
-  /* We just have to create the singleon here */
-  clutter_device_manager_get_default ();
+  backend->device_manager = g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_EVDEV,
+                                          "backend", backend,
+                                          NULL);
 }
 
 void
 _clutter_events_evdev_uninit (ClutterBackend *backend)
 {
-  ClutterDeviceManager *manager;
-
-  manager = clutter_device_manager_get_default ();
-  g_object_unref (manager);
+  CLUTTER_NOTE (EVENT, "Uninitializing evdev backend");
 }
