@@ -42,6 +42,14 @@
 
 static void clutter_stage_window_iface_init (ClutterStageWindowIface *iface);
 
+enum
+{
+  PROP_0,
+
+  PROP_BACKEND,
+  PROP_WRAPPER
+};
+
 G_DEFINE_TYPE_WITH_CODE (ClutterStageWin32,
 			 clutter_stage_win32,
 			 G_TYPE_OBJECT,
@@ -79,17 +87,17 @@ clutter_stage_win32_hide (ClutterStageWindow *stage_window)
 }
 
 static void
-clutter_stage_win32_get_geometry (ClutterStageWindow *stage_window,
-                                  ClutterGeometry    *geometry)
+clutter_stage_win32_get_geometry (ClutterStageWindow    *stage_window,
+                                  cairo_rectangle_int_t *geometry)
 {
   ClutterStageWin32 *stage_win32 = CLUTTER_STAGE_WIN32 (stage_window);
 
-  if ((stage_win32->state & CLUTTER_STAGE_STATE_FULLSCREEN))
+  if (_clutter_stage_is_fullscreen (stage_win32->wrapper))
     {
-      geometry->width = (stage_win32->fullscreen_rect.right
-                         - stage_win32->fullscreen_rect.left);
-      geometry->height = (stage_win32->fullscreen_rect.bottom
-                          - stage_win32->fullscreen_rect.top);
+      geometry->width = stage_win32->fullscreen_rect.right
+                      - stage_win32->fullscreen_rect.left;
+      geometry->height = stage_win32->fullscreen_rect.bottom
+                       - stage_win32->fullscreen_rect.top;
       return;
     }
 
@@ -169,7 +177,7 @@ clutter_stage_win32_resize (ClutterStageWindow *stage_window,
   if (width != stage_win32->win_width || height != stage_win32->win_height)
     {
       /* Ignore size requests if we are in full screen mode */
-      if ((stage_win32->state & CLUTTER_STAGE_STATE_FULLSCREEN) == 0)
+      if (!_clutter_stage_is_fullscreen (stage_win32->wrapper))
         {
           stage_win32->win_width = width;
           stage_win32->win_height = height;
@@ -264,7 +272,7 @@ get_window_style (ClutterStageWin32 *stage_win32)
   ClutterStage *wrapper = stage_win32->wrapper;
 
   /* Fullscreen mode shouldn't have any borders */
-  if ((stage_win32->state & CLUTTER_STAGE_STATE_FULLSCREEN))
+  if (_clutter_stage_is_fullscreen (wrapper))
     return WS_POPUP;
   /* Otherwise it's an overlapped window but if it isn't resizable
      then it shouldn't have a thick frame */
@@ -315,11 +323,6 @@ clutter_stage_win32_set_fullscreen (ClutterStageWindow *stage_window,
   LONG old_style = GetWindowLongW (hwnd, GWL_STYLE);
   ClutterStageStateEvent event;
 
-  if (value)
-    stage_win32->state |= CLUTTER_STAGE_STATE_FULLSCREEN;
-  else
-    stage_win32->state &= ~CLUTTER_STAGE_STATE_FULLSCREEN;
-
   if (hwnd)
     {
       /* Update the window style but preserve the visibility */
@@ -356,12 +359,18 @@ clutter_stage_win32_set_fullscreen (ClutterStageWindow *stage_window,
     }
 
   /* Report the state change */
-  memset (&event, 0, sizeof (event));
-  event.type = CLUTTER_STAGE_STATE;
-  event.stage = CLUTTER_STAGE (stage_win32->wrapper);
-  event.new_state = stage_win32->state;
-  event.changed_mask = CLUTTER_STAGE_STATE_FULLSCREEN;
-  clutter_event_put ((ClutterEvent *) &event);
+  if (value)
+    {
+      _clutter_stage_update_state (stage_win32->wrapper,
+                                   0,
+                                   CLUTTER_STAGE_STATE_FULLSCREEN);
+    }
+  else
+    {
+      _clutter_stage_update_state (stage_win32->wrapper,
+                                   CLUTTER_STAGE_STATE_FULLSCREEN,
+                                   0);
+    }
 }
 
 static ATOM
@@ -398,7 +407,6 @@ clutter_stage_win32_realize (ClutterStageWindow *stage_window)
   gfloat width;
   gfloat height;
   GError *error = NULL;
-  const char *clutter_vblank;
 
   CLUTTER_NOTE (MISC, "Realizing main stage");
 
@@ -424,7 +432,7 @@ clutter_stage_win32_realize (ClutterStageWindow *stage_window)
 
       /* If we're in fullscreen mode then use the fullscreen rect
          instead */
-      if ((stage_win32->state & CLUTTER_STAGE_STATE_FULLSCREEN))
+      if (_clutter_stage_is_fullscreen (stage_win32->wrapper))
         {
           get_fullscreen_rect (stage_win32);
           win_xpos = stage_win32->fullscreen_rect.left;
@@ -470,9 +478,8 @@ clutter_stage_win32_realize (ClutterStageWindow *stage_window)
   cogl_win32_onscreen_set_foreign_window (stage_win32->onscreen,
                                           stage_win32->hwnd);
 
-  clutter_vblank = _clutter_backend_win32_get_vblank ();
-  if (clutter_vblank && strcmp (clutter_vblank, "none") == 0)
-    cogl_onscreen_set_swap_throttled (stage_win32->onscreen, FALSE);
+  cogl_onscreen_set_swap_throttled (stage_win32->onscreen,
+                                    _clutter_get_sync_to_vblank ());
 
   framebuffer = COGL_FRAMEBUFFER (stage_win32->onscreen);
   if (!cogl_framebuffer_allocate (framebuffer, &error))
@@ -538,6 +545,37 @@ clutter_stage_win32_redraw (ClutterStageWindow *stage_window)
     cogl_framebuffer_swap_buffers (COGL_FRAMEBUFFER (stage_win32->onscreen));
 }
 
+static CoglFramebuffer *
+clutter_stage_win32_get_active_framebuffer (ClutterStageWindow *stage_window)
+{
+  ClutterStageWin32 *stage_win32 = CLUTTER_STAGE_WIN32 (stage_window);
+
+  return COGL_FRAMEBUFFER (stage_win32->onscreen);
+}
+
+static void
+clutter_stage_win32_set_property (GObject      *gobject,
+                                  guint         prop_id,
+                                  const GValue *value,
+                                  GParamSpec   *pspec)
+{
+  ClutterStageWin32 *self = CLUTTER_STAGE_WIN32 (gobject);
+
+  switch (prop_id)
+    {
+    case PROP_BACKEND:
+      self->backend = g_value_get_object (value);
+      break;
+
+    case PROP_WRAPPER:
+      self->wrapper = g_value_get_object (value);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
+    }
+}
+
 static void
 clutter_stage_win32_dispose (GObject *gobject)
 {
@@ -563,7 +601,11 @@ clutter_stage_win32_class_init (ClutterStageWin32Class *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
+  gobject_class->set_property = clutter_stage_win32_set_property;
   gobject_class->dispose = clutter_stage_win32_dispose;
+
+  g_object_class_override_property (gobject_class, PROP_BACKEND, "backend");
+  g_object_class_override_property (gobject_class, PROP_WRAPPER, "wrapper");
 }
 
 static void
@@ -598,6 +640,7 @@ clutter_stage_window_iface_init (ClutterStageWindowIface *iface)
   iface->realize = clutter_stage_win32_realize;
   iface->unrealize = clutter_stage_win32_unrealize;
   iface->redraw = clutter_stage_win32_redraw;
+  iface->get_active_framebuffer = clutter_stage_win32_get_active_framebuffer;
 }
 
 /**
@@ -673,7 +716,7 @@ clutter_win32_get_stage_from_window (HWND hwnd)
 
 typedef struct {
   ClutterStageWin32 *stage_win32;
-  ClutterGeometry geom;
+  cairo_rectangle_int_t geom;
   HWND hwnd;
   guint destroy_old_hwnd : 1;
 } ForeignWindowData;
@@ -700,7 +743,7 @@ set_foreign_window_callback (ClutterActor *actor,
   fwd->stage_win32->win_width = fwd->geom.width;
   fwd->stage_win32->win_height = fwd->geom.height;
 
-  clutter_actor_set_geometry (actor, &fwd->geom);
+  clutter_actor_set_size (actor, fwd->geom.width, fwd->geom.height);
 
   /* calling this with the stage unrealized will unset the stage
    * from the GL context; once the stage is realized the GL context
@@ -733,9 +776,13 @@ clutter_win32_set_stage_foreign (ClutterStage *stage,
   g_return_val_if_fail (CLUTTER_IS_STAGE (stage), FALSE);
   g_return_val_if_fail (hwnd != NULL, FALSE);
 
-  actor = CLUTTER_ACTOR (stage);
-
   impl = _clutter_stage_get_window (stage);
+  if (!CLUTTER_IS_STAGE_WIN32 (impl))
+    {
+      g_critical ("The Clutter backend is not a Windows backend");
+      return FALSE;
+    }
+
   stage_win32 = CLUTTER_STAGE_WIN32 (impl);
 
   if (!GetClientRect (hwnd, &client_rect))
@@ -758,6 +805,8 @@ clutter_win32_set_stage_foreign (ClutterStage *stage,
   fwd.geom.width = client_rect.right - client_rect.left;
   fwd.geom.height = client_rect.bottom - client_rect.top;
 
+  actor = CLUTTER_ACTOR (stage);
+
   _clutter_actor_rerealize (actor,
                             set_foreign_window_callback,
                             &fwd);
