@@ -59,7 +59,6 @@ static const char *clutter_input_axis_atom_names[] = {
   "Abs Tilt X",         /* CLUTTER_INPUT_AXIS_XTILT */
   "Abs Tilt Y",         /* CLUTTER_INPUT_AXIS_YTILT */
   "Abs Wheel",          /* CLUTTER_INPUT_AXIS_WHEEL */
-  "Abs Distance",       /* CLUTTER_INPUT_AXIS_DISTANCE */
 };
 
 #define N_AXIS_ATOMS    G_N_ELEMENTS (clutter_input_axis_atom_names)
@@ -94,13 +93,13 @@ translate_valuator_class (Display             *xdisplay,
       atoms_initialized = TRUE;
     }
 
-  for (i = 0;
-       i < N_AXIS_ATOMS;
+  for (i = CLUTTER_INPUT_AXIS_IGNORE;
+       i <= CLUTTER_INPUT_AXIS_WHEEL;
        i += 1)
     {
       if (clutter_input_axis_atoms[i] == class->label)
         {
-          axis = i + 1;
+          axis = i;
           break;
         }
     }
@@ -155,166 +154,24 @@ translate_device_classes (Display             *xdisplay,
                                     (XIValuatorClassInfo *) class_info);
           break;
 
-#ifdef HAVE_XINPUT_2_2
-        case XIScrollClass:
-          {
-            XIScrollClassInfo *scroll_info = (XIScrollClassInfo *) class_info;
-            ClutterScrollDirection direction;
-
-            if (scroll_info->scroll_type == XIScrollTypeVertical)
-              direction = CLUTTER_SCROLL_DOWN;
-            else
-              direction = CLUTTER_SCROLL_RIGHT;
-
-            CLUTTER_NOTE (BACKEND, "Scroll valuator %d: %s, increment: %f",
-                          scroll_info->number,
-                          scroll_info->scroll_type == XIScrollTypeVertical
-                            ? "vertical"
-                            : "horizontal",
-                          scroll_info->increment);
-
-            _clutter_input_device_add_scroll_info (device,
-                                                   scroll_info->number,
-                                                   direction,
-                                                   scroll_info->increment);
-          }
-          break;
-#endif /* HAVE_XINPUT_2_2 */
-
         default:
           break;
         }
     }
 }
 
-static gboolean
-is_touch_device (XIAnyClassInfo         **classes,
-                 guint                    n_classes,
-                 ClutterInputDeviceType  *device_type,
-                 guint                   *n_touch_points)
-{
-#ifdef HAVE_XINPUT_2_2
-  guint i;
-
-  for (i = 0; i < n_classes; i++)
-    {
-      XITouchClassInfo *class = (XITouchClassInfo *) classes[i];
-
-      if (class->type != XITouchClass)
-        continue;
-
-      if (class->num_touches > 0)
-        {
-          if (class->mode == XIDirectTouch)
-            *device_type = CLUTTER_TOUCHSCREEN_DEVICE;
-          else if (class->mode == XIDependentTouch)
-            *device_type = CLUTTER_TOUCHPAD_DEVICE;
-          else
-            continue;
-
-          *n_touch_points = class->num_touches;
-
-          return TRUE;
-        }
-    }
-#endif
-
-  return FALSE;
-}
-
-static gboolean
-is_touchpad_device (ClutterBackendX11 *backend_x11,
-                    XIDeviceInfo      *info)
-{
-  gulong nitems, bytes_after;
-  guint32 *data = NULL;
-  int rc, format;
-  Atom type;
-  Atom prop;
-
-  prop = XInternAtom (backend_x11->xdpy, "libinput Tapping Enabled", True);
-  if (prop == None)
-    return FALSE;
-
-  clutter_x11_trap_x_errors ();
-  rc = XIGetProperty (backend_x11->xdpy,
-                      info->deviceid,
-                      prop,
-                      0, 1, False, XA_INTEGER, &type, &format, &nitems, &bytes_after,
-                      (guchar **) &data);
-  clutter_x11_untrap_x_errors ();
-
-  /* We don't care about the data */
-  XFree (data);
-
-  if (rc != Success || type != XA_INTEGER || format != 8 || nitems != 1)
-    return FALSE;
-
-  return TRUE;
-}
-
-static gboolean
-get_device_ids (ClutterBackendX11  *backend_x11,
-                XIDeviceInfo       *info,
-                gchar             **vendor_id,
-                gchar             **product_id)
-{
-  gulong nitems, bytes_after;
-  guint32 *data = NULL;
-  int rc, format;
-  Atom type;
-
-  clutter_x11_trap_x_errors ();
-  rc = XIGetProperty (backend_x11->xdpy,
-                      info->deviceid,
-                      XInternAtom (backend_x11->xdpy, "Device Product ID", False),
-                      0, 2, False, XA_INTEGER, &type, &format, &nitems, &bytes_after,
-                      (guchar **) &data);
-  clutter_x11_untrap_x_errors ();
-
-  if (rc != Success || type != XA_INTEGER || format != 32 || nitems != 2)
-    {
-      XFree (data);
-      return FALSE;
-    }
-
-  if (vendor_id)
-    *vendor_id = g_strdup_printf ("%.4x", data[0]);
-  if (product_id)
-    *product_id = g_strdup_printf ("%.4x", data[1]);
-
-  XFree (data);
-
-  return TRUE;
-}
-
 static ClutterInputDevice *
 create_device (ClutterDeviceManagerXI2 *manager_xi2,
                ClutterBackendX11       *backend_x11,
                XIDeviceInfo            *info)
 {
-  ClutterInputDeviceType source, touch_source;
+  ClutterInputDeviceType source;
   ClutterInputDevice *retval;
   ClutterInputMode mode;
   gboolean is_enabled;
-  guint num_touches = 0;
-  gchar *vendor_id = NULL, *product_id = NULL;
 
   if (info->use == XIMasterKeyboard || info->use == XISlaveKeyboard)
-    {
-      source = CLUTTER_KEYBOARD_DEVICE;
-    }
-  else if (is_touchpad_device (backend_x11, info))
-    {
-      source = CLUTTER_TOUCHPAD_DEVICE;
-    }
-  else if (info->use == XISlavePointer &&
-           is_touch_device (info->classes, info->num_classes,
-                            &touch_source,
-                            &num_touches))
-    {
-      source = touch_source;
-    }
+    source = CLUTTER_KEYBOARD_DEVICE;
   else
     {
       gchar *name;
@@ -327,8 +184,6 @@ create_device (ClutterDeviceManagerXI2 *manager_xi2,
         source = CLUTTER_CURSOR_DEVICE;
       else if (strstr (name, "wacom") != NULL || strstr (name, "pen") != NULL)
         source = CLUTTER_PEN_DEVICE;
-      else if (strstr (name, "touchpad") != NULL)
-        source = CLUTTER_TOUCHPAD_DEVICE;
       else
         source = CLUTTER_POINTER_DEVICE;
 
@@ -356,10 +211,6 @@ create_device (ClutterDeviceManagerXI2 *manager_xi2,
       break;
     }
 
-  if (info->use != XIMasterKeyboard &&
-      info->use != XIMasterPointer)
-    get_device_ids (backend_x11, info, &vendor_id, &product_id);
-
   retval = g_object_new (CLUTTER_TYPE_INPUT_DEVICE_XI2,
                          "name", info->name,
                          "id", info->deviceid,
@@ -369,15 +220,11 @@ create_device (ClutterDeviceManagerXI2 *manager_xi2,
                          "device-mode", mode,
                          "backend", backend_x11,
                          "enabled", is_enabled,
-                         "vendor-id", vendor_id,
-                         "product-id", product_id,
                          NULL);
 
   translate_device_classes (backend_x11->xdpy, retval,
                             info->classes,
                             info->num_classes);
-  g_free (vendor_id);
-  g_free (product_id);
 
   CLUTTER_NOTE (BACKEND, "Created device '%s' (id: %d, has-cursor: %s)",
                 info->name,
@@ -402,7 +249,7 @@ add_device (ClutterDeviceManagerXI2 *manager_xi2,
    */
   g_hash_table_replace (manager_xi2->devices_by_id,
                         GINT_TO_POINTER (info->deviceid),
-                        device);
+                        g_object_ref (device));
 
   if (info->use == XIMasterPointer ||
       info->use == XIMasterKeyboard)
@@ -484,25 +331,17 @@ translate_hierarchy_event (ClutterBackendX11       *backend_x11,
 
   for (i = 0; i < ev->num_info; i++)
     {
-      if (ev->info[i].flags & XIDeviceEnabled &&
-          !g_hash_table_lookup (manager_xi2->devices_by_id,
-                                GINT_TO_POINTER (ev->info[i].deviceid)))
+      if (ev->info[i].flags & XIDeviceEnabled)
         {
           XIDeviceInfo *info;
           int n_devices;
 
           CLUTTER_NOTE (EVENT, "Hierarchy event: device enabled");
 
-          clutter_x11_trap_x_errors ();
           info = XIQueryDevice (backend_x11->xdpy,
                                 ev->info[i].deviceid,
                                 &n_devices);
-          clutter_x11_untrap_x_errors ();
-          if (info != NULL)
-            {
-              add_device (manager_xi2, backend_x11, &info[0], FALSE);
-              XIFreeDeviceInfo (info);
-            }
+          add_device (manager_xi2, backend_x11, &info[0], FALSE);
         }
       else if (ev->info[i].flags & XIDeviceDisabled)
         {
@@ -516,7 +355,6 @@ translate_hierarchy_event (ClutterBackendX11       *backend_x11,
           ClutterInputDevice *master, *slave;
           XIDeviceInfo *info;
           int n_devices;
-          gboolean send_changed = FALSE;
 
           CLUTTER_NOTE (EVENT, "Hierarchy event: slave %s",
                         (ev->info[i].flags & XISlaveAttached)
@@ -532,40 +370,20 @@ translate_hierarchy_event (ClutterBackendX11       *backend_x11,
             {
               _clutter_input_device_remove_slave (master, slave);
               _clutter_input_device_set_associated_device (slave, NULL);
-
-              send_changed = TRUE;
             }
 
           /* and attach the slave to the new master if needed */
           if (ev->info[i].flags & XISlaveAttached)
             {
-              clutter_x11_trap_x_errors ();
               info = XIQueryDevice (backend_x11->xdpy,
                                     ev->info[i].deviceid,
                                     &n_devices);
-              clutter_x11_untrap_x_errors ();
-              if (info != NULL)
-                {
-                  master = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                                GINT_TO_POINTER (info->attachment));
-                  if (master != NULL)
-                    {
-                      _clutter_input_device_set_associated_device (slave, master);
-                      _clutter_input_device_add_slave (master, slave);
+              master = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                            GINT_TO_POINTER (info->attachment));
+              _clutter_input_device_set_associated_device (slave, master);
+              _clutter_input_device_add_slave (master, slave);
 
-                      send_changed = TRUE;
-                    }
-                  XIFreeDeviceInfo (info);
-                }
-            }
-
-          if (send_changed)
-            {
-              ClutterStage *stage = _clutter_input_device_get_stage (master);
-              if (stage != NULL)
-                _clutter_stage_x11_events_device_changed (CLUTTER_STAGE_X11 (_clutter_stage_get_window (stage)), 
-                                                          master,
-                                                          CLUTTER_DEVICE_MANAGER (manager_xi2));
+              XIFreeDeviceInfo (info);
             }
         }
     }
@@ -596,11 +414,6 @@ get_event_stage (ClutterEventTranslator *translator,
     case XI_ButtonPress:
     case XI_ButtonRelease:
     case XI_Motion:
-#ifdef HAVE_XINPUT_2_2
-    case XI_TouchBegin:
-    case XI_TouchUpdate:
-    case XI_TouchEnd:
-#endif /* HAVE_XINPUT_2_2 */
       {
         XIDeviceEvent *xev = (XIDeviceEvent *) xi_event;
 
@@ -687,6 +500,7 @@ static gdouble *
 translate_axes (ClutterInputDevice *device,
                 gdouble             x,
                 gdouble             y,
+                ClutterStageX11    *stage_x11,
                 XIValuatorState    *valuators)
 {
   guint n_axes = clutter_input_device_get_n_axes (device);
@@ -727,112 +541,6 @@ translate_axes (ClutterInputDevice *device,
   return retval;
 }
 
-static void
-translate_coords (ClutterStageX11 *stage_x11,
-                  gdouble          event_x,
-                  gdouble          event_y,
-                  gfloat          *x_out,
-                  gfloat          *y_out)
-{
-  ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_x11);
-  ClutterActor *stage = CLUTTER_ACTOR (stage_cogl->wrapper);
-  gfloat stage_width;
-  gfloat stage_height;
-
-  clutter_actor_get_size (stage, &stage_width, &stage_height);
-
-  *x_out = CLAMP (event_x / stage_x11->scale_factor, 0, stage_width);
-  *y_out = CLAMP (event_y / stage_x11->scale_factor, 0, stage_height);
-}
-
-static gdouble
-scroll_valuators_changed (ClutterInputDevice *device,
-                          XIValuatorState    *valuators,
-                          gdouble            *dx_p,
-                          gdouble            *dy_p)
-{
-  gboolean retval = FALSE;
-  guint n_axes, n_val, i;
-  double *values;
-
-  n_axes = clutter_input_device_get_n_axes (device);
-  values = valuators->values;
-
-  *dx_p = *dy_p = 0.0;
-
-  n_val = 0;
-
-  for (i = 0; i < MIN (valuators->mask_len * 8, n_axes); i++)
-    {
-      ClutterScrollDirection direction;
-      gdouble delta;
-
-      if (!XIMaskIsSet (valuators->mask, i))
-        continue;
-
-      if (_clutter_input_device_get_scroll_delta (device, i,
-                                                  values[n_val],
-                                                  &direction,
-                                                  &delta))
-        {
-          retval = TRUE;
-
-          if (direction == CLUTTER_SCROLL_UP ||
-              direction == CLUTTER_SCROLL_DOWN)
-            *dy_p = delta;
-          else
-            *dx_p = delta;
-        }
-
-      n_val += 1;
-    }
-
-  return retval;
-}
-
-static void
-clutter_device_manager_xi2_select_stage_events (ClutterDeviceManager *manager,
-                                                ClutterStage         *stage)
-{
-  ClutterBackendX11 *backend_x11;
-  ClutterStageX11 *stage_x11;
-  XIEventMask xi_event_mask;
-  unsigned char *mask;
-  int len;
-
-  backend_x11 = CLUTTER_BACKEND_X11 (clutter_get_default_backend ());
-  stage_x11 = CLUTTER_STAGE_X11 (_clutter_stage_get_window (stage));
-
-  len = XIMaskLen (XI_LASTEVENT);
-  mask = g_new0 (unsigned char, len);
-
-  XISetMask (mask, XI_Motion);
-  XISetMask (mask, XI_ButtonPress);
-  XISetMask (mask, XI_ButtonRelease);
-  XISetMask (mask, XI_KeyPress);
-  XISetMask (mask, XI_KeyRelease);
-  XISetMask (mask, XI_Enter);
-  XISetMask (mask, XI_Leave);
-
-#ifdef HAVE_XINPUT_2_2
-  /* enable touch event support if we're running on XInput 2.2 */
-  if (backend_x11->xi_minor >= 2)
-    {
-      XISetMask (mask, XI_TouchBegin);
-      XISetMask (mask, XI_TouchUpdate);
-      XISetMask (mask, XI_TouchEnd);
-    }
-#endif /* HAVE_XINPUT_2_2 */
-
-  xi_event_mask.deviceid = XIAllMasterDevices;
-  xi_event_mask.mask = mask;
-  xi_event_mask.mask_len = len;
-
-  XISelectEvents (backend_x11->xdpy, stage_x11->xwin, &xi_event_mask, 1);
-
-  g_free (mask);
-}
-
 static ClutterTranslateReturn
 clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
                                             gpointer                native,
@@ -892,19 +600,11 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
 
         device = g_hash_table_lookup (manager_xi2->devices_by_id,
                                       GINT_TO_POINTER (xev->deviceid));
-        source_device = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                             GINT_TO_POINTER (xev->sourceid));
-        if (device)
-          {
-            _clutter_input_device_reset_axes (device);
-            translate_device_classes (backend_x11->xdpy,
-                                      device,
-                                      xev->classes,
-                                      xev->num_classes);
-          }
-
-        if (source_device)
-          _clutter_input_device_reset_scroll_info (source_device);
+        _clutter_input_device_reset_axes (device);
+        translate_device_classes (backend_x11->xdpy,
+                                  device,
+                                  xev->classes,
+                                  xev->num_classes);
       }
       retval = CLUTTER_TRANSLATE_REMOVE;
       break;
@@ -923,14 +623,15 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
 
         event->key.time = xev->time;
         event->key.stage = stage;
-	_clutter_input_device_xi2_translate_state (event, &xev->mods, &xev->buttons, &xev->group);
+        event->key.modifier_state =
+          _clutter_input_device_xi2_translate_state (&xev->mods, &xev->buttons);
         event->key.hardware_keycode = xev->detail;
 
           /* keyval is the key ignoring all modifiers ('1' vs. '!') */
         event->key.keyval =
           _clutter_keymap_x11_translate_key_state (backend_x11->keymap,
                                                    event->key.hardware_keycode,
-                                                   &event->key.modifier_state,
+                                                   event->key.modifier_state,
                                                    NULL);
 
         /* KeyEvents have platform specific data associated to them */
@@ -994,27 +695,12 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
       {
         XIDeviceEvent *xev = (XIDeviceEvent *) xi_event;
 
-        source_device = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                             GINT_TO_POINTER (xev->sourceid));
-        device = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                      GINT_TO_POINTER (xev->deviceid));
-
-        /* Set the stage for core events coming out of nowhere (see bug #684509) */
-        if (clutter_input_device_get_device_mode (device) == CLUTTER_INPUT_MODE_MASTER &&
-            clutter_input_device_get_pointer_stage (device) == NULL &&
-            stage != NULL)
-          _clutter_input_device_set_stage (device, stage);
-
         switch (xev->detail)
           {
           case 4:
           case 5:
           case 6:
           case 7:
-            /* we only generate Scroll events on ButtonPress */
-            if (xi_event->evtype == XI_ButtonRelease)
-              return CLUTTER_TRANSLATE_REMOVE;
-
             event->scroll.type = event->type = CLUTTER_SCROLL;
 
             if (xev->detail == 4)
@@ -1029,42 +715,26 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
             event->scroll.stage = stage;
 
             event->scroll.time = xev->time;
-            translate_coords (stage_x11, xev->event_x, xev->event_y, &event->scroll.x, &event->scroll.y);
-	    _clutter_input_device_xi2_translate_state (event,
-						       &xev->mods,
-						       &xev->buttons,
-						       &xev->group);
+            event->scroll.x = xev->event_x;
+            event->scroll.y = xev->event_y;
+            event->scroll.modifier_state =
+              _clutter_input_device_xi2_translate_state (&xev->mods,
+                                                         &xev->buttons);
 
+            source_device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                                 GINT_TO_POINTER (xev->sourceid));
             clutter_event_set_source_device (event, source_device);
+
+            device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                          GINT_TO_POINTER (xev->deviceid));
             clutter_event_set_device (event, device);
 
             event->scroll.axes = translate_axes (event->scroll.device,
                                                  event->scroll.x,
                                                  event->scroll.y,
+                                                 stage_x11,
                                                  &xev->valuators);
 
-            CLUTTER_NOTE (EVENT,
-                          "scroll: win:0x%x, device:%d '%s', time:%d "
-                          "(direction:%s, "
-                          "x:%.2f, y:%.2f, "
-                          "emulated:%s)",
-                          (unsigned int) stage_x11->xwin,
-                          device->id,
-                          device->device_name,
-                          event->any.time,
-                          event->scroll.direction == CLUTTER_SCROLL_UP ? "up" :
-                          event->scroll.direction == CLUTTER_SCROLL_DOWN ? "down" :
-                          event->scroll.direction == CLUTTER_SCROLL_LEFT ? "left" :
-                          event->scroll.direction == CLUTTER_SCROLL_RIGHT ? "right" :
-                          "invalid",
-                          event->scroll.x,
-                          event->scroll.y,
-#ifdef HAVE_XINPUT_2_2
-                          (xev->flags & XIPointerEmulated) ? "yes" : "no"
-#else
-                          "no"
-#endif
-                          );
             break;
 
           default:
@@ -1075,54 +745,43 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
             event->button.stage = stage;
 
             event->button.time = xev->time;
-            translate_coords (stage_x11, xev->event_x, xev->event_y, &event->button.x, &event->button.y);
+            event->button.x = xev->event_x;
+            event->button.y = xev->event_y;
             event->button.button = xev->detail;
-	    _clutter_input_device_xi2_translate_state (event,
-						       &xev->mods,
-						       &xev->buttons,
-						       &xev->group);
+            event->button.modifier_state =
+              _clutter_input_device_xi2_translate_state (&xev->mods,
+                                                         &xev->buttons);
 
+            source_device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                                 GINT_TO_POINTER (xev->sourceid));
             clutter_event_set_source_device (event, source_device);
+
+            device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                          GINT_TO_POINTER (xev->deviceid));
             clutter_event_set_device (event, device);
 
             event->button.axes = translate_axes (event->button.device,
                                                  event->button.x,
                                                  event->button.y,
+                                                 stage_x11,
                                                  &xev->valuators);
-
-            CLUTTER_NOTE (EVENT,
-                          "%s: win:0x%x, device:%d '%s', time:%d "
-                          "(button:%d, "
-                          "x:%.2f, y:%.2f, "
-                          "axes:%s, "
-                          "emulated:%s)",
-                          event->any.type == CLUTTER_BUTTON_PRESS
-                            ? "button press  "
-                            : "button release",
-                          (unsigned int) stage_x11->xwin,
-                          device->id,
-                          device->device_name,
-                          event->any.time,
-                          event->button.button,
-                          event->button.x,
-                          event->button.y,
-                          event->button.axes != NULL ? "yes" : "no",
-#ifdef HAVE_XINPUT_2_2
-                          (xev->flags & XIPointerEmulated) ? "yes" : "no"
-#else
-                          "no"
-#endif
-                          );
             break;
           }
 
         if (source_device != NULL && device->stage != NULL)
           _clutter_input_device_set_stage (source_device, device->stage);
 
-#ifdef HAVE_XINPUT_2_2
-        if (xev->flags & XIPointerEmulated)
-          _clutter_event_set_pointer_emulated (event, TRUE);
-#endif /* HAVE_XINPUT_2_2 */
+        CLUTTER_NOTE (EVENT,
+                      "%s: win:0x%x, device:%s (button:%d, x:%.2f, y:%.2f, axes:%s)",
+                      event->any.type == CLUTTER_BUTTON_PRESS
+                        ? "button press  "
+                        : "button release",
+                      (unsigned int) stage_x11->xwin,
+                      event->button.device->device_name,
+                      event->button.button,
+                      event->button.x,
+                      event->button.y,
+                      event->button.axes != NULL ? "yes" : "no");
 
         if (xi_event->evtype == XI_ButtonPress)
           _clutter_stage_x11_set_user_time (stage_x11, event->button.time);
@@ -1134,81 +793,37 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
     case XI_Motion:
       {
         XIDeviceEvent *xev = (XIDeviceEvent *) xi_event;
-        gdouble delta_x, delta_y;
-
-        source_device = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                             GINT_TO_POINTER (xev->sourceid));
-        device = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                      GINT_TO_POINTER (xev->deviceid));
-
-        /* Set the stage for core events coming out of nowhere (see bug #684509) */
-        if (clutter_input_device_get_device_mode (device) == CLUTTER_INPUT_MODE_MASTER &&
-            clutter_input_device_get_pointer_stage (device) == NULL &&
-            stage != NULL)
-          _clutter_input_device_set_stage (device, stage);
-
-        if (scroll_valuators_changed (source_device,
-                                      &xev->valuators,
-                                      &delta_x, &delta_y))
-          {
-            event->scroll.type = event->type = CLUTTER_SCROLL;
-            event->scroll.direction = CLUTTER_SCROLL_SMOOTH;
-
-            event->scroll.stage = stage;
-            event->scroll.time = xev->time;
-            translate_coords (stage_x11, xev->event_x, xev->event_y, &event->scroll.x, &event->scroll.y);
-	    _clutter_input_device_xi2_translate_state (event,
-						       &xev->mods,
-						       &xev->buttons,
-						       &xev->group);
-
-            clutter_event_set_scroll_delta (event, delta_x, delta_y);
-            clutter_event_set_source_device (event, source_device);
-            clutter_event_set_device (event, device);
-
-            CLUTTER_NOTE (EVENT,
-                          "smooth scroll: win:0x%x device:%d '%s' (x:%.2f, y:%.2f, delta:%f, %f)",
-                          (unsigned int) stage_x11->xwin,
-                          event->scroll.device->id,
-                          event->scroll.device->device_name,
-                          event->scroll.x,
-                          event->scroll.y,
-                          delta_x, delta_y);
-
-            retval = CLUTTER_TRANSLATE_QUEUE;
-            break;
-          }
 
         event->motion.type = event->type = CLUTTER_MOTION;
 
         event->motion.stage = stage;
 
         event->motion.time = xev->time;
-        translate_coords (stage_x11, xev->event_x, xev->event_y, &event->motion.x, &event->motion.y);
-	_clutter_input_device_xi2_translate_state (event,
-						   &xev->mods,
-						   &xev->buttons,
-						   &xev->group);
+        event->motion.x = xev->event_x;
+        event->motion.y = xev->event_y;
+        event->motion.modifier_state =
+          _clutter_input_device_xi2_translate_state (&xev->mods,
+                                                     &xev->buttons);
 
+        source_device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                             GINT_TO_POINTER (xev->sourceid));
         clutter_event_set_source_device (event, source_device);
+
+        device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                      GINT_TO_POINTER (xev->deviceid));
         clutter_event_set_device (event, device);
 
         event->motion.axes = translate_axes (event->motion.device,
                                              event->motion.x,
                                              event->motion.y,
+                                             stage_x11,
                                              &xev->valuators);
 
         if (source_device != NULL && device->stage != NULL)
           _clutter_input_device_set_stage (source_device, device->stage);
 
-#ifdef HAVE_XINPUT_2_2
-        if (xev->flags & XIPointerEmulated)
-          _clutter_event_set_pointer_emulated (event, TRUE);
-#endif /* HAVE_XINPUT_2_2 */
-
-        CLUTTER_NOTE (EVENT, "motion: win:0x%x device:%d '%s' (x:%.2f, y:%.2f, axes:%s)",
+        CLUTTER_NOTE (EVENT, "motion: win:0x%x device:%s (x:%.2f, y:%.2f, axes:%s)",
                       (unsigned int) stage_x11->xwin,
-                      event->motion.device->id,
                       event->motion.device->device_name,
                       event->motion.x,
                       event->motion.y,
@@ -1218,120 +833,6 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
       }
       break;
 
-#ifdef HAVE_XINPUT_2_2
-    case XI_TouchBegin:
-      {
-        XIDeviceEvent *xev = (XIDeviceEvent *) xi_event;
-        device = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                      GINT_TO_POINTER (xev->deviceid));
-        if (!_clutter_input_device_get_stage (device))
-          _clutter_input_device_set_stage (device, stage);
-      }
-      /* Fall through */
-    case XI_TouchEnd:
-      {
-        XIDeviceEvent *xev = (XIDeviceEvent *) xi_event;
-
-        source_device = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                             GINT_TO_POINTER (xev->sourceid));
-
-        if (xi_event->evtype == XI_TouchBegin)
-          event->touch.type = event->type = CLUTTER_TOUCH_BEGIN;
-        else
-          event->touch.type = event->type = CLUTTER_TOUCH_END;
-
-        event->touch.stage = stage;
-        event->touch.time = xev->time;
-        translate_coords (stage_x11, xev->event_x, xev->event_y, &event->touch.x, &event->touch.y);
-	_clutter_input_device_xi2_translate_state (event,
-						   &xev->mods,
-						   &xev->buttons,
-						   &xev->group);
-
-        clutter_event_set_source_device (event, source_device);
-
-        device = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                      GINT_TO_POINTER (xev->deviceid));
-        clutter_event_set_device (event, device);
-
-        event->touch.axes = translate_axes (event->touch.device,
-                                            event->motion.x,
-                                            event->motion.y,
-                                            &xev->valuators);
-
-        if (xi_event->evtype == XI_TouchBegin)
-          {
-            event->touch.modifier_state |= CLUTTER_BUTTON1_MASK;
-
-            _clutter_stage_x11_set_user_time (stage_x11, event->touch.time);
-          }
-
-        event->touch.sequence = GUINT_TO_POINTER (xev->detail);
-
-        if (xev->flags & XITouchEmulatingPointer)
-          _clutter_event_set_pointer_emulated (event, TRUE);
-
-        CLUTTER_NOTE (EVENT, "touch %s: win:0x%x device:%d '%s' (seq:%d, x:%.2f, y:%.2f, axes:%s)",
-                      event->type == CLUTTER_TOUCH_BEGIN ? "begin" : "end",
-                      (unsigned int) stage_x11->xwin,
-                      event->touch.device->id,
-                      event->touch.device->device_name,
-                      GPOINTER_TO_UINT (event->touch.sequence),
-                      event->touch.x,
-                      event->touch.y,
-                      event->touch.axes != NULL ? "yes" : "no");
-
-        retval = CLUTTER_TRANSLATE_QUEUE;
-      }
-      break;
-
-    case XI_TouchUpdate:
-      {
-        XIDeviceEvent *xev = (XIDeviceEvent *) xi_event;
-
-        source_device = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                             GINT_TO_POINTER (xev->sourceid));
-
-        event->touch.type = event->type = CLUTTER_TOUCH_UPDATE;
-        event->touch.stage = stage;
-        event->touch.time = xev->time;
-        event->touch.sequence = GUINT_TO_POINTER (xev->detail);
-        translate_coords (stage_x11, xev->event_x, xev->event_y, &event->touch.x, &event->touch.y);
-
-        clutter_event_set_source_device (event, source_device);
-
-        device = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                      GINT_TO_POINTER (xev->deviceid));
-        clutter_event_set_device (event, device);
-
-        event->touch.axes = translate_axes (event->touch.device,
-                                            event->motion.x,
-                                            event->motion.y,
-                                            &xev->valuators);
-
-	_clutter_input_device_xi2_translate_state (event,
-						   &xev->mods,
-						   &xev->buttons,
-						   &xev->group);
-        event->touch.modifier_state |= CLUTTER_BUTTON1_MASK;
-
-        if (xev->flags & XITouchEmulatingPointer)
-          _clutter_event_set_pointer_emulated (event, TRUE);
-
-        CLUTTER_NOTE (EVENT, "touch update: win:0x%x device:%d '%s' (seq:%d, x:%.2f, y:%.2f, axes:%s)",
-                      (unsigned int) stage_x11->xwin,
-                      event->touch.device->id,
-                      event->touch.device->device_name,
-                      GPOINTER_TO_UINT (event->touch.sequence),
-                      event->touch.x,
-                      event->touch.y,
-                      event->touch.axes != NULL ? "yes" : "no");
-
-        retval = CLUTTER_TRANSLATE_QUEUE;
-      }
-      break;
-#endif /* HAVE_XINPUT_2_2 */
-
     case XI_Enter:
     case XI_Leave:
       {
@@ -1352,9 +853,10 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
             event->crossing.related = NULL;
 
             event->crossing.time = xev->time;
-            translate_coords (stage_x11, xev->event_x, xev->event_y, &event->crossing.x, &event->crossing.y);
+            event->crossing.x = xev->event_x;
+            event->crossing.y = xev->event_y;
 
-            _clutter_input_device_set_stage (device, stage);
+            _clutter_stage_add_device (stage, device);
           }
         else
           {
@@ -1375,13 +877,12 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
             event->crossing.related = NULL;
 
             event->crossing.time = xev->time;
-            translate_coords (stage_x11, xev->event_x, xev->event_y, &event->crossing.x, &event->crossing.y);
+            event->crossing.x = xev->event_x;
+            event->crossing.y = xev->event_y;
 
-            _clutter_input_device_set_stage (device, NULL);
+            _clutter_stage_remove_device (stage, device);
           }
 
-        _clutter_input_device_reset_scroll_info (source_device);
-
         clutter_event_set_device (event, device);
         clutter_event_set_source_device (event, source_device);
 
@@ -1454,29 +955,25 @@ clutter_device_manager_xi2_get_core_device (ClutterDeviceManager   *manager,
                                             ClutterInputDeviceType  device_type)
 {
   ClutterDeviceManagerXI2 *manager_xi2 = CLUTTER_DEVICE_MANAGER_XI2 (manager);
-  ClutterInputDevice *pointer = NULL;
-  GList *l;
+  ClutterBackendX11 *backend_x11;
+  ClutterInputDevice *device;
+  int device_id;
 
-  for (l = manager_xi2->master_devices; l != NULL ; l = l->next)
-    {
-      ClutterInputDevice *device = l->data;
-      if (clutter_input_device_get_device_type (device) == CLUTTER_POINTER_DEVICE)
-        {
-          pointer = device;
-          break;
-        }
-    }
+  backend_x11 =
+    CLUTTER_BACKEND_X11 (_clutter_device_manager_get_backend (manager));
 
-  if (pointer == NULL)
-    return NULL;
+  XIGetClientPointer (backend_x11->xdpy, None, &device_id);
+
+  device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                GINT_TO_POINTER (device_id));
 
   switch (device_type)
     {
     case CLUTTER_POINTER_DEVICE:
-      return pointer;
+      return device;
 
     case CLUTTER_KEYBOARD_DEVICE:
-      return clutter_input_device_get_associated_device (pointer);
+      return clutter_input_device_get_associated_device (device);
 
     default:
       break;
@@ -1508,8 +1005,8 @@ relate_slaves (gpointer key,
   ClutterDeviceManagerXI2 *manager_xi2 = data;
   ClutterInputDevice *master, *slave;
 
-  slave = g_hash_table_lookup (manager_xi2->devices_by_id, key);
-  master = g_hash_table_lookup (manager_xi2->devices_by_id, value);
+  master = g_hash_table_lookup (manager_xi2->devices_by_id, key);
+  slave = g_hash_table_lookup (manager_xi2->devices_by_id, value);
 
   _clutter_input_device_set_associated_device (slave, master);
   _clutter_input_device_add_slave (master, slave);
@@ -1539,9 +1036,6 @@ clutter_device_manager_xi2_constructed (GObject *gobject)
     {
       XIDeviceInfo *xi_device = &info[i];
 
-      if (!xi_device->enabled)
-        continue;
-
       add_device (manager_xi2, backend_x11, xi_device, TRUE);
 
       if (xi_device->use == XIMasterPointer ||
@@ -1579,8 +1073,6 @@ clutter_device_manager_xi2_constructed (GObject *gobject)
                                             clutter_x11_get_root_window (),
                                             &event_mask);
 
-  XSync (backend_x11->xdpy, False);
-
   if (G_OBJECT_CLASS (clutter_device_manager_xi2_parent_class)->constructed)
     G_OBJECT_CLASS (clutter_device_manager_xi2_parent_class)->constructed (gobject);
 }
@@ -1631,7 +1123,6 @@ clutter_device_manager_xi2_class_init (ClutterDeviceManagerXI2Class *klass)
   manager_class->get_devices = clutter_device_manager_xi2_get_devices;
   manager_class->get_core_device = clutter_device_manager_xi2_get_core_device;
   manager_class->get_device = clutter_device_manager_xi2_get_device;
-  manager_class->select_stage_events = clutter_device_manager_xi2_select_stage_events;
 }
 
 static void
