@@ -341,12 +341,10 @@ clutter_x11_texture_pixmap_real_queue_damage_redraw (
                                               gint height)
 {
   ClutterX11TexturePixmapPrivate *priv = texture->priv;
-  ClutterActor    *self = CLUTTER_ACTOR (texture);
-  ClutterActorBox  allocation;
-  float            scale_x;
-  float            scale_y;
-  ClutterVertex    origin;
-  ClutterPaintVolume clip;
+  ClutterActor *self = CLUTTER_ACTOR (texture);
+  ClutterActorBox allocation;
+  float scale_x, scale_y;
+  cairo_rectangle_int_t clip;
 
   /* NB: clutter_actor_queue_clipped_redraw expects a box in the actor's
    * coordinate space so we need to convert from pixmap coordinates to
@@ -373,17 +371,11 @@ clutter_x11_texture_pixmap_real_queue_damage_redraw (
   scale_x = (allocation.x2 - allocation.x1) / priv->pixmap_width;
   scale_y = (allocation.y2 - allocation.y1) / priv->pixmap_height;
 
-  _clutter_paint_volume_init_static (&clip, self);
-
-  origin.x = x * scale_x;
-  origin.y = y * scale_y;
-  origin.z = 0;
-  clutter_paint_volume_set_origin (&clip, &origin);
-  clutter_paint_volume_set_width (&clip, width * scale_x);
-  clutter_paint_volume_set_height (&clip, height * scale_y);
-
-  _clutter_actor_queue_redraw_with_clip (self, 0, &clip);
-  clutter_paint_volume_free (&clip);
+  clip.x = x * scale_x;
+  clip.y = y * scale_y;
+  clip.width = width * scale_x;
+  clip.height = height * scale_y;
+  clutter_actor_queue_redraw_with_clip (self, &clip);
 }
 
 static void
@@ -529,7 +521,6 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
   GObjectClass      *object_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
   GParamSpec        *pspec;
-  ClutterBackend    *default_backend;
 
   g_type_class_add_private (klass, sizeof (ClutterX11TexturePixmapPrivate));
 
@@ -714,15 +705,6 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
   g_signal_override_class_handler ("queue-damage-redraw",
                                    CLUTTER_X11_TYPE_TEXTURE_PIXMAP,
                                    G_CALLBACK (clutter_x11_texture_pixmap_real_queue_damage_redraw));
-
-  default_backend = clutter_get_default_backend ();
-
-  if (!CLUTTER_IS_BACKEND_X11 (default_backend))
-    {
-      g_critical ("ClutterX11TexturePixmap instantiated with a "
-                  "non-X11 backend");
-      return;
-    }
 }
 
 static void
@@ -954,13 +936,17 @@ clutter_x11_texture_pixmap_set_window (ClutterX11TexturePixmap *texture,
 {
   ClutterX11TexturePixmapPrivate *priv;
   XWindowAttributes attr;
-  Display *dpy = clutter_x11_get_default_display ();
+  Display *dpy;
 
   g_return_if_fail (CLUTTER_X11_IS_TEXTURE_PIXMAP (texture));
 
   if (!clutter_x11_has_composite_extension ())
     return;
 
+  dpy = clutter_x11_get_default_display ();
+  if (dpy == NULL)
+    return;
+
 #if HAVE_XCOMPOSITE
   priv = texture->priv;
 
@@ -1058,7 +1044,7 @@ clutter_x11_texture_pixmap_sync_window_internal (ClutterX11TexturePixmap *textur
   priv->window_height = height;
   priv->override_redirect = override_redirect;
 
-  if (!clutter_x11_has_composite_extension())
+  if (!clutter_x11_has_composite_extension ())
     {
       /* FIXME: this should just be an error, this is unlikely to work worth anything */
       clutter_x11_texture_pixmap_set_pixmap (texture, priv->window);
@@ -1148,12 +1134,15 @@ clutter_x11_texture_pixmap_sync_window (ClutterX11TexturePixmap *texture)
   if (priv->destroyed)
     return;
 
-  if (priv->window)
+  if (priv->window != None)
     {
-      XWindowAttributes attr;
       Display *dpy = clutter_x11_get_default_display ();
+      XWindowAttributes attr;
       Status status;
 
+      if (dpy == NULL)
+        return;
+
       clutter_x11_trap_x_errors ();
 
       status = XGetWindowAttributes (dpy, priv->window, &attr);
