@@ -71,14 +71,22 @@ test_material_with_primitives (TestState *state,
                                int x, int y,
                                guint32 color)
 {
-  CoglTextureVertex verts[4] = {
-    { .x = 0,          .y = 0,          .z = 0 },
-    { .x = 0,          .y = QUAD_WIDTH, .z = 0 },
-    { .x = QUAD_WIDTH, .y = QUAD_WIDTH, .z = 0 },
-    { .x = QUAD_WIDTH, .y = 0,          .z = 0 },
-  };
+  CoglTextureVertex verts[4];
   CoglHandle vbo;
 
+  verts[0].x = 0;
+  verts[0].y = 0;
+  verts[0].z = 0;
+  verts[1].x = 0;
+  verts[1].y = QUAD_WIDTH;
+  verts[1].z = 0;
+  verts[2].x = QUAD_WIDTH;
+  verts[2].y = QUAD_WIDTH;
+  verts[2].z = 0;
+  verts[3].x = QUAD_WIDTH;
+  verts[3].y = 0;
+  verts[3].z = 0;
+
   cogl_push_matrix ();
 
   cogl_translate (x * QUAD_WIDTH, y * QUAD_WIDTH, 0);
@@ -311,7 +319,7 @@ queue_redraw (gpointer stage)
 {
   clutter_actor_queue_redraw (CLUTTER_ACTOR (stage));
 
-  return TRUE;
+  return G_SOURCE_CONTINUE;
 }
 
 void
@@ -325,7 +333,7 @@ test_cogl_materials (TestConformSimpleFixture *fixture,
 
   test_conform_get_gl_functions (&gl_functions);
 
-  stage = clutter_stage_get_default ();
+  stage = clutter_stage_new ();
 
   clutter_stage_set_color (CLUTTER_STAGE (stage), &stage_color);
   clutter_actor_get_geometry (stage, &state.stage_geom);
@@ -336,7 +344,7 @@ test_cogl_materials (TestConformSimpleFixture *fixture,
   /* We force continuous redrawing of the stage, since we need to skip
    * the first few frames, and we wont be doing anything else that
    * will trigger redrawing. */
-  idle_source = g_idle_add (queue_redraw, stage);
+  idle_source = clutter_threads_add_idle (queue_redraw, stage);
 
   g_signal_connect (group, "paint", G_CALLBACK (on_paint), &state);
 
@@ -346,7 +354,8 @@ test_cogl_materials (TestConformSimpleFixture *fixture,
 
   g_source_remove (idle_source);
 
+  clutter_actor_destroy (stage);
+
   if (g_test_verbose ())
     g_print ("OK\n");
 }
-
